import os
from pathlib import Path
from typing import Optional

import pandas as pd
import torch
import torch.nn.functional as F
from GANDLF.compute import inference_loop
from GANDLF.utils import get_unique_timestamp


def InferenceManager(
    dataframe: pd.DataFrame,
    modelDir: str,
    parameters: dict,
    device: str,
    outputDir: Optional[str] = None,
) -> None:
    """
    This function is used to perform inference on a model using a dataframe.

    Args:
        dataframe (pd.DataFrame): The dataframe containing the data to be used for inference.
        modelDir (str): The path to the model directory.
        parameters (dict): The parameters to be used for inference.
        device (str): The device to be used for inference.
        outputDir (Optional[str], optional): The output directory for the inference results. Defaults to None.
    """
    if outputDir is None:
        outputDir = os.path.join(modelDir, get_unique_timestamp())
        print(
            "Output directory not provided, creating a new directory with a unique timestamp: ",
            outputDir,
        )
    Path(outputDir).mkdir(parents=True, exist_ok=True)

    parameters["output_dir"] = outputDir

    # initialize parameters for inference
    for key in ["penalty_weights", "sampling_weights", "class_weights"]:
        parameters.setdefault(key, None)

    n_folds = parameters["nested_training"]["validation"]
    modelDir_split = modelDir.split(",") if "," in modelDir else [modelDir]

    averaged_probs_list = []
    for current_modelDir in modelDir_split:
        fold_dirs = (
            [
                os.path.join(current_modelDir, d, "")
                for d in sorted(os.listdir(current_modelDir))
                if d.isdigit()
            ]
            if n_folds > 1
            else [current_modelDir]
        )

        probs_list = []
        is_classification = parameters["problem_type"] == "classification"
        parameters["model"].setdefault("type", "torch")
        class_list = (
            [str(c) for c in parameters["model"]["class_list"]]
            if is_classification
            else None
        )

        for fold_dir in fold_dirs:
            parameters["current_fold_dir"] = fold_dir
            inference_loop(
                inferenceDataFromPickle=dataframe,
                modelDir=fold_dir,
                device=device,
                parameters=parameters,
                outputDir=outputDir,
            )

            if is_classification:
                logits_path = os.path.join(fold_dir, "logits.csv")
                if os.path.isfile(logits_path):
                    fold_logits = pd.read_csv(logits_path)[class_list].values
                    fold_logits = torch.from_numpy(fold_logits)
                    fold_probs = F.softmax(fold_logits, dim=1)
                    probs_list.append(fold_probs)

        if is_classification and probs_list:
            probs_list = torch.stack(probs_list)
            averaged_probs_list.append(torch.mean(probs_list, 0))

    # this logic should be changed if we want to do multi-fold inference for histo images
    if averaged_probs_list and is_classification:
        averaged_probs_df = pd.DataFrame(
            columns=["SubjectID", "PredictedClass"] + class_list
        )
        averaged_probs_df["SubjectID"] = dataframe.iloc[:, 0]

        averaged_probs_across_models = torch.mean(
            torch.stack(averaged_probs_list), 0
        ).numpy()
        averaged_probs_df[class_list] = averaged_probs_across_models
        averaged_probs_df["PredictedClass"] = [
            class_list[idx] for idx in averaged_probs_across_models.argmax(axis=1)
        ]

        filepath_to_save = os.path.join(outputDir, "final_preds_and_avg_probs.csv")
        if os.path.isfile(filepath_to_save):
            filepath_to_save = os.path.join(
                outputDir,
                "final_preds_and_avg_probs_" + get_unique_timestamp() + ".csv",
            )

        averaged_probs_df.to_csv(filepath_to_save, index=False)



from typing import Optional
import torch

import PIL.Image
import numpy as np

from torchio.transforms.intensity_transform import IntensityTransform
from torchio.data.subject import Subject


class RGB2RGBA(IntensityTransform):
    """
    Convert RGB image to RGBA image.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def apply_transform(self, subject: Subject) -> Subject:
        for image in self.get_images(subject):
            image_data_to_set = image.data
            # only proceed with RGB is detected
            if image_data_to_set.shape[0] == 3:
                image_data_array = image_data_to_set.numpy()
                if image_data_array.shape[-1] == 1:
                    image_data_array = image_data_array[..., 0]
                    image_data_array = image_data_array.transpose([1, 2, 0])
                image_pil = PIL.Image.fromarray(image_data_array.astype(np.uint8))
                image_pil_rgb = image_pil.convert("RGBA")
                image_data_to_set = torch.from_numpy(
                    np.array(image_pil_rgb).transpose([2, 0, 1])
                ).unsqueeze(-1)
            image.set_data(image_data_to_set)
        return subject


class RGBA2RGB(IntensityTransform):
    """
    Convert RGBA image to RGB image.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def apply_transform(self, subject: Subject) -> Subject:
        for image in self.get_images(subject):
            image_data_to_set = image.data
            # only proceed with RGBA is detected
            if image_data_to_set.shape[0] == 4:
                image_data_array = image_data_to_set.numpy()
                if image_data_array.shape[-1] == 1:
                    image_data_array = image_data_array[..., 0]
                    image_data_array = image_data_array.transpose([1, 2, 0])
                image_pil = PIL.Image.fromarray(image_data_array.astype(np.uint8))
                image_pil_rgb = image_pil.convert("RGB")
                image_data_to_set = torch.from_numpy(
                    np.array(image_pil_rgb).transpose([2, 0, 1])
                ).unsqueeze(-1)
            image.set_data(image_data_to_set)
        return subject


def rgba2rgb_transform(parameters: Optional[dict] = None) -> RGBA2RGB:
    """
    This function returns the transform to convert RGBA to RGB.

    Args:
        parameters (dict, optional): The parameters for the transform. Defaults to None.

    Returns:
        RGBA2RGB: The transform to convert RGBA to RGB.
    """
    return RGBA2RGB()


def rgb2rgba_transform(parameters: Optional[dict] = None) -> RGB2RGBA:
    """
    This function returns the transform to convert RGB to RGBA.

    Args:
        parameters (dict, optional): The parameters for the transform. Defaults to None.

    Returns:
        RGB2RGBA: The transform to convert RGB to RGBA.
    """
    return RGB2RGBA()



from typing import List, Optional, Tuple, Union
import os, pathlib, math, copy
from enum import Enum
import numpy as np
import SimpleITK as sitk
import torchio
import cv2

from .generic import get_filename_extension_sanitized


def resample_image(
    input_image: sitk.Image,
    spacing: Union[np.ndarray, List[float], Tuple[float]],
    size: Optional[Union[np.ndarray, List[float], Tuple[float]]] = None,
    interpolator: Optional[Enum] = sitk.sitkLinear,
    outsideValue: Optional[int] = 0,
) -> sitk.Image:
    """
    This function resamples the input image based on the spacing and size.

    Args:
        input_image (sitk.Image): The input image to be resampled.
        spacing (Union[np.ndarray, List[float], Tuple[float]]): The desired spacing for the resampled image.
        size (Optional[Union[np.ndarray, List[float], Tuple[float]]], optional): The desired size for the resampled image. Defaults to None.
        interpolator (Optional[Enum], optional): The desired interpolator. Defaults to sitk.sitkLinear.
        outsideValue (Optional[int], optional): The value to be used for the outside of the image. Defaults to 0.

    Returns:
        sitk.Image: The resampled image.
    """
    assert (
        len(spacing) == input_image.GetDimension()
    ), "The spacing dimension is inconsistent with the input dataset, please check parameters."

    # Set Size
    if size is None:
        inSpacing = input_image.GetSpacing()
        inSize = input_image.GetSize()
        size = [
            int(math.ceil(inSize[i] * (inSpacing[i] / spacing[i])))
            for i in range(input_image.GetDimension())
        ]

    assert (
        len(size) == input_image.GetDimension()
    ), "The size dimension is inconsistent with the input dataset, please check parameters."

    # Resample input image
    return sitk.Resample(
        input_image,
        size,
        sitk.Transform(),
        interpolator,
        input_image.GetOrigin(),
        spacing,
        input_image.GetDirection(),
        outsideValue,
    )


def resize_image(
    input_image: sitk.Image,
    output_size: Union[np.ndarray, list, tuple],
    interpolator: Optional[Enum] = sitk.sitkLinear,
) -> sitk.Image:
    """
    This function resizes the input image based on the output size.

    Args:
        input_image (sitk.Image): The input image to be resized.
        output_size (Union[np.ndarray, list, tuple]): The desired output size for the resized image.
        interpolator (Optional[Enum], optional): The desired interpolator. Defaults to sitk.sitkLinear.

    Returns:
        sitk.Image: The resized image.
    """
    output_size_parsed = None
    inputSize = input_image.GetSize()
    inputSpacing = np.array(input_image.GetSpacing())
    outputSpacing = np.array(inputSpacing)

    output_size_parsed = output_size
    if isinstance(output_size, dict):
        if "resize" in output_size:
            output_size_parsed = output_size["resize"]

    assert len(output_size_parsed) == len(
        inputSpacing
    ), "The output size dimension is inconsistent with the input dataset, please check parameters."

    for i, n in enumerate(output_size_parsed):
        outputSpacing[i] = outputSpacing[i] * (inputSize[i] / n)

    return resample_image(input_image, outputSpacing, interpolator=interpolator)


def softer_sanity_check(
    base_property: Union[np.ndarray, List[float], Tuple[float]],
    new_property: Union[np.ndarray, List[float], Tuple[float]],
    threshold: Optional[float] = 0.00001,
) -> bool:
    """
    This function performs a softer sanity check on the input properties.

    Args:
        base_property (Union[np.ndarray, List[float], Tuple[float]]): The base property.
        new_property (Union[np.ndarray, List[float], Tuple[float]]): The new property.
        threshold (Optional[float], optional): The threshold for comparison. Defaults to 0.00001.

    Returns:
        bool: True if the properties are consistent within the threshold.
    """
    arr_1 = np.array(base_property)
    arr_2 = np.array(new_property)
    diff = np.sum(arr_1 - arr_2)

    result = False
    if diff <= threshold:
        result = True

    return result


def perform_sanity_check_on_subject(subject: torchio.Subject, parameters: dict) -> bool:
    """
    This function performs a sanity check on the image modalities in input subject to ensure that they are consistent.

    Args:
        subject (torchio.Subject): The input subject.
        parameters (dict): The parameters passed by the user yaml.

    Returns:
        bool: True if the sanity check passes.
    """
    # read the first image and save that for comparison
    file_reader_base = None

    list_for_comparison = copy.deepcopy(parameters["headers"]["channelHeaders"])
    if parameters["headers"]["labelHeader"] is not None:
        list_for_comparison.append("label")

    def _get_itkimage_or_filereader(
        subject_str_key: Union[str, sitk.Image]
    ) -> Union[sitk.ImageFileReader, sitk.Image]:
        """
        Helper function to get the itk image or file reader from the subject.

        Args:
            subject_str_key (Union[str, sitk.Image]): The subject string key or itk image.

        Returns:
            Union[sitk.ImageFileReader, sitk.Image]: The itk image or file reader.
        """
        if subject_str_key["path"] != "":
            file_reader = sitk.ImageFileReader()
            file_reader.SetFileName(subject_str_key["path"])
            file_reader.ReadImageInformation()
            return file_reader
        else:
            # this case is required if any tensor/imaging operation has been applied in dataloader
            file_reader = subject_str_key.as_sitk()

    if len(list_for_comparison) > 1:
        for key in list_for_comparison:
            if file_reader_base is None:
                file_reader_base = _get_itkimage_or_filereader(subject[str(key)])
            else:
                file_reader_current = _get_itkimage_or_filereader(subject[str(key)])

                # this check needs to be absolute
                assert (
                    file_reader_base.GetDimension()
                    == file_reader_current.GetDimension()
                ), (
                    "Dimensions for Subject '"
                    + subject["subject_id"]
                    + "' are not consistent."
                )

                # other checks can be softer
                assert softer_sanity_check(
                    file_reader_base.GetOrigin(), file_reader_current.GetOrigin()
                ), (
                    "Origin for Subject '"
                    + subject["subject_id"]
                    + "' are not consistent."
                )

                assert softer_sanity_check(
                    file_reader_base.GetDirection(), file_reader_current.GetDirection()
                ), (
                    "Orientation for Subject '"
                    + subject["subject_id"]
                    + "' are not consistent."
                )

                assert softer_sanity_check(
                    file_reader_base.GetSpacing(), file_reader_current.GetSpacing()
                ), (
                    "Spacing for Subject '"
                    + subject["subject_id"]
                    + "' are not consistent."
                )

    return True


def write_training_patches(subject: torchio.Subject, params: dict) -> None:
    """
    This function writes the training patches to disk.

    Args:
        subject (torchio.Subject): The input subject.
        params (dict): The parameters passed by the user yaml; needs the "output_dir" and "current_epoch" keys to be present.
    """
    # create folder tree for saving the patches
    training_output_dir = os.path.join(params["output_dir"], "training_patches")
    pathlib.Path(training_output_dir).mkdir(parents=True, exist_ok=True)
    training_output_dir_epoch = os.path.join(
        training_output_dir, str(params["current_epoch"])
    )
    pathlib.Path(training_output_dir_epoch).mkdir(parents=True, exist_ok=True)
    training_output_dir_current_subject = os.path.join(
        training_output_dir_epoch, subject["subject_id"][0]
    )
    pathlib.Path(training_output_dir_current_subject).mkdir(parents=True, exist_ok=True)

    # write the training patches to disk
    ext = get_filename_extension_sanitized(subject["path_to_metadata"][0])
    for key in params["channel_keys"]:
        img_to_write = torchio.ScalarImage(
            tensor=subject[key][torchio.DATA][0], affine=subject[key]["affine"][0]
        ).as_sitk()
        sitk.WriteImage(
            img_to_write,
            os.path.join(training_output_dir_current_subject, "modality_" + key + ext),
        )

    if params["label_keys"] is not None:
        img_to_write = torchio.ScalarImage(
            tensor=subject[params["label_keys"][0]][torchio.DATA][0],
            affine=subject[key]["affine"][0],
        ).as_sitk()
        sitk.WriteImage(
            img_to_write,
            os.path.join(training_output_dir_current_subject, "label_" + key + ext),
        )


def get_correct_padding_size(
    patch_size: Union[List[int], Tuple[int]], model_dimension: int
):
    """
    This function returns the correct padding size based on the patch size and overlap.

    Args:
        patch_size (Union[List[int], Tuple[int]]): The patch size.
        model_dimension (int): The model dimension.

    Returns:
        Union[list, tuple]: The correct padding size.
    """
    psize_pad = list(np.asarray(np.ceil(np.divide(patch_size, 2)), dtype=int))
    # ensure that the patch size for z-axis is not 1 for 2d images
    if model_dimension == 2:
        psize_pad[-1] = 0 if psize_pad[-1] == 1 else psize_pad[-1]

    return psize_pad


def applyCustomColorMap(im_gray: np.ndarray) -> np.ndarray:
    """
    Internal function to apply a custom color map to the input image.

    Args:
        im_gray (np.ndarray): The input image.

    Returns:
        np.ndarray: The image with the custom color map applied.
    """
    img_bgr = cv2.cvtColor(im_gray.astype(np.uint8), cv2.COLOR_BGR2RGB)
    lut = np.zeros((256, 1, 3), dtype=np.uint8)
    lut[:, 0, 0] = np.zeros((256)).tolist()
    lut[:, 0, 1] = np.zeros((256)).tolist()
    lut[:, 0, 2] = np.arange(0, 256, 1).tolist()
    return cv2.LUT(img_bgr, lut)



import pandas as pd
import os, pickle, shutil
from pathlib import Path
from torch.profiler import profile, ProfilerActivity

from GANDLF.compute import training_loop
from GANDLF.utils import get_dataframe, split_data

import yaml


def TrainingManager(
    dataframe: pd.DataFrame,
    outputDir: str,
    parameters: dict,
    device: str,
    resume: bool,
    reset: bool,
) -> None:
    """
    This is the training manager that ties all the training functionality together

    Args:
        dataframe (pandas.DataFrame): The full data from CSV.
        outputDir (str): The main output directory.
        parameters (dict): The parameters dictionary.
        device (str): The device to perform computations on.
        resume (bool): Whether the previous run will be resumed or not.
        reset (bool): Whether the previous run will be reset or not.
    """
    if reset:
        shutil.rmtree(outputDir)
        Path(outputDir).mkdir(parents=True, exist_ok=True)

    # save the current model configuration as a sanity check
    currentModelConfigPickle = os.path.join(outputDir, "parameters.pkl")
    if (not os.path.exists(currentModelConfigPickle)) or reset or resume:
        with open(currentModelConfigPickle, "wb") as handle:
            pickle.dump(parameters, handle, protocol=pickle.HIGHEST_PROTOCOL)
    else:
        if os.path.exists(currentModelConfigPickle):
            print(
                "Using previously saved parameter file",
                currentModelConfigPickle,
                flush=True,
            )
            parameters = pickle.load(open(currentModelConfigPickle, "rb"))

    dataframe_split = split_data(dataframe, parameters)

    last_indeces, _, _, _ = dataframe_split[-1]

    # check the last indeces to see if single fold training is requested
    singleFoldTesting = True if last_indeces[0] == 0 else False
    singleFoldValidation = True if last_indeces[1] == 0 else False

    for (
        testing_and_valid_indeces,
        trainingData,
        validationData,
        testingData,
    ) in dataframe_split:
        # the output of the current fold is only needed if multi-fold training is happening
        currentTestingOutputFolder = outputDir
        if not singleFoldTesting:
            currentTestingOutputFolder = os.path.join(
                outputDir, "testing_" + str(testing_and_valid_indeces[0])
            )
            Path(currentTestingOutputFolder).mkdir(parents=True, exist_ok=True)

        currentValidationOutputFolder = currentTestingOutputFolder
        if not singleFoldValidation:
            currentValidationOutputFolder = os.path.join(
                currentTestingOutputFolder, str(testing_and_valid_indeces[1])
            )
            Path(currentValidationOutputFolder).mkdir(parents=True, exist_ok=True)

        # initialize the dataframes and save them to disk
        data_dict = {
            "training": trainingData,
            "validation": validationData,
            "testing": testingData,
        }
        data_dict_files = {}
        for data_type, data in data_dict.items():
            data_dict_files[data_type] = None
            if data is not None:
                currentDataPickle = os.path.join(
                    currentValidationOutputFolder, "data_" + data_type + ".pkl"
                )
                data_dict_files[data_type] = currentDataPickle
                if (not os.path.exists(currentDataPickle)) or reset or resume:
                    data.to_pickle(currentDataPickle)
                    data.to_csv(currentDataPickle.replace(".pkl", ".csv"), index=False)
                else:
                    # read the data from the pickle if present
                    data_dict[data_type] = get_dataframe(currentDataPickle)

        # parallel_compute_command is an empty string, thus no parallel computing requested
        if not parameters["parallel_compute_command"]:
            training_loop(
                training_data=data_dict["training"],
                validation_data=data_dict["validation"],
                output_dir=currentValidationOutputFolder,
                device=device,
                params=parameters,
                testing_data=data_dict["testing"],
            )

        else:
            # call hpc command here
            parallel_compute_command_actual = parameters[
                "parallel_compute_command"
            ].replace("${outputDir}", currentValidationOutputFolder)

            assert (
                "python" in parallel_compute_command_actual
            ), "The 'parallel_compute_command_actual' needs to have the python from the virtual environment, which is usually '${GANDLF_dir}/venv/bin/python'"

            command = (
                parallel_compute_command_actual
                + " -m GANDLF.training_loop -train_loader_pickle "
                + data_dict_files["training"]
                + " -val_loader_pickle "
                + data_dict_files["validation"]
                + " -parameter_pickle "
                + currentModelConfigPickle
                + " -device "
                + str(device)
                + " -outputDir "
                + currentValidationOutputFolder
                + " -testing_loader_pickle "
                + data_dict_files["testing"]
            )

            print("Running command: ", command, flush=True)
            os.system(command, flush=True)


def TrainingManager_split(
    dataframe_train: pd.DataFrame,
    dataframe_validation: pd.DataFrame,
    dataframe_testing: pd.DataFrame,
    outputDir: str,
    parameters: dict,
    device: str,
    resume: bool,
    reset: bool,
    _profile: bool,
):
    """
    This is the training manager that ties all the training functionality together

    Args:
        dataframe_train (pd.DataFrame): The training data from CSV.
        dataframe_validation (pd.DataFrame): The validation data from CSV.
        dataframe_testing (pd.DataFrame): The testing data from CSV.
        outputDir (str): The main output directory.
        parameters (dict): The parameters dictionary.
        device (str): The device to perform computations on.
        resume (bool): Whether the previous run will be resumed or not.
        reset (bool): Whether the previous run will be reset or not.
        _profile(bool):Whether the we want the profile activity or not.

    """
    currentModelConfigPickle = os.path.join(outputDir, "parameters.pkl")
    currentModelConfigYaml = os.path.join(outputDir, "config.yaml")

    if (not os.path.exists(currentModelConfigPickle)) or reset or resume:
        with open(currentModelConfigPickle, "wb") as handle:
            pickle.dump(parameters, handle, protocol=pickle.HIGHEST_PROTOCOL)
    else:
        if os.path.exists(currentModelConfigPickle):
            print(
                "Using previously saved parameter file",
                currentModelConfigPickle,
                flush=True,
            )
            parameters = pickle.load(open(currentModelConfigPickle, "rb"))

    if (not os.path.exists(currentModelConfigYaml)) or reset or resume:
        with open(currentModelConfigYaml, "w") as handle:
            yaml.dump(parameters, handle, default_flow_style=False)

    if _profile:
        with profile(
            activities=[ProfilerActivity.CPU, ProfilerActivity.CUDA],
            profile_memory=True,
            record_shapes=True,
        ) as prof:
            training_loop(
                training_data=dataframe_train,
                validation_data=dataframe_validation,
                output_dir=outputDir,
                device=device,
                params=parameters,
                testing_data=dataframe_testing,
            )
        print(prof.key_averages().table(sort_by="cpu_time_total", row_limit=10))
    else:
        training_loop(
            training_data=dataframe_train,
            validation_data=dataframe_validation,
            output_dir=outputDir,
            device=device,
            params=parameters,
            testing_data=dataframe_testing,
        )



from typing import Optional, Union
from .config_manager import ConfigManager


def parseConfig(
    config_file_path: Union[str, dict], version_check_flag: Optional[bool] = True
) -> None:
    """
    This function parses the configuration file and returns a dictionary of parameters.

    Args:
        config_file_path (Union[str, dict]): The filename of the configuration file.
        version_check_flag (bool, optional): Whether to check the version in configuration file. Defaults to True.

    Returns:
        dict: The parameter dictionary.
    """
    print(
        "WARNING: `GANDLF.parseConfig` will be deprecated in favor of `GANDLF.config_manager` in a future version."
    )
    return ConfigManager(config_file_path, version_check_flag)



#!/usr/bin/env python

"""The setup script."""


import sys, re, os
from setuptools import setup, find_packages


try:
    with open("README.md") as readme_file:
        readme = readme_file.read()
except Exception as error:
    readme = "No README information found."
    sys.stderr.write("Warning: Could not open '%s' due %s\n" % ("README.md", error))

try:
    filepath = "GANDLF/version.py"
    version_file = open(filepath)
    (__version__,) = re.findall('__version__ = "(.*)"', version_file.read())

except Exception as error:
    __version__ = "0.0.1"
    sys.stderr.write("Warning: Could not open '%s' due %s\n" % (filepath, error))

# Handle cases where specific files need to be bundled into the final package as installed via PyPI
dockerfiles = [
    item
    for item in os.listdir(os.path.dirname(os.path.abspath(__file__)))
    if (os.path.isfile(item) and item.startswith("Dockerfile-"))
]

# Any extra files should be located at `GANDLF` module folder (not in repo root)
extra_files = ["logging_config.yaml"]
toplevel_package_excludes = ["testing*"]

# specifying version for `black` separately because it is also used to [check for lint](https://github.com/mlcommons/GaNDLF/blob/master/.github/workflows/black.yml)
black_version = "23.11.0"
requirements = [
    "torch==2.5.0",
    f"black=={black_version}",
    "numpy==1.25.0",
    "scipy",
    "SimpleITK!=2.0.*",
    "SimpleITK!=2.2.1",  # https://github.com/mlcommons/GaNDLF/issues/536
    "torchvision",
    "tqdm",
    "torchio==0.19.6",
    "pandas>=2.0.0",
    "scikit-learn>=0.23.2",
    "scikit-image>=0.19.1",
    "setuptools",
    "seaborn",
    "pyyaml==6.0.1",
    "matplotlib",
    "gdown==5.1.0",
    "pytest",
    "coverage",
    "pytest-cov",
    "psutil",
    "medcam",
    "opencv-python",
    "torchmetrics==1.1.2",
    "zarr==2.10.3",
    "pydicom",
    "onnx",
    "torchinfo==1.7.0",
    "segmentation-models-pytorch==0.3.3",
    "ACSConv==0.1.1",
    # https://github.com/docker/docker-py/issues/3256
    "requests>=2.32.2",
    "docker",
    "dicom-anonymizer==1.0.12",
    "twine",
    "zarr",
    "keyring",
    "monai==1.3.0",
    "click>=8.0.0",
    "deprecated",
    "packaging==24.0",
    "typer==0.9.0",
    "colorlog",
    "opacus==1.5.2",
    "huggingface-hub==0.25.1",
    "openslide-bin",
    "openslide-python==1.4.1",
    "lion-pytorch==0.2.2",
]

if __name__ == "__main__":
    setup(
        name="GANDLF",
        version=__version__,
        author="MLCommons",
        author_email="gandlf@mlcommons.org",
        python_requires=">3.8, <3.12",
        packages=find_packages(
            where=os.path.dirname(os.path.abspath(__file__)),
            exclude=toplevel_package_excludes,
        ),
        entry_points={
            "console_scripts": [
                "gandlf=GANDLF.entrypoints.cli_tool:gandlf",
                # old entrypoints
                "gandlf_run=GANDLF.entrypoints.run:old_way",
                "gandlf_constructCSV=GANDLF.entrypoints.construct_csv:old_way",
                "gandlf_collectStats=GANDLF.entrypoints.collect_stats:old_way",
                "gandlf_patchMiner=GANDLF.entrypoints.patch_miner:old_way",
                "gandlf_preprocess=GANDLF.entrypoints.preprocess:old_way",
                "gandlf_anonymizer=GANDLF.entrypoints.anonymizer:old_way",
                "gandlf_configGenerator=GANDLF.entrypoints.config_generator:old_way",
                "gandlf_verifyInstall=GANDLF.entrypoints.verify_install:old_way",
                "gandlf_recoverConfig=GANDLF.entrypoints.recover_config:old_way",
                "gandlf_deploy=GANDLF.entrypoints.deploy:old_way",
                "gandlf_optimizeModel=GANDLF.entrypoints.optimize_model:old_way",
                "gandlf_generateMetrics=GANDLF.entrypoints.generate_metrics:old_way",
                "gandlf_debugInfo=GANDLF.entrypoints.debug_info:old_way",
                "gandlf_splitCSV=GANDLF.entrypoints.split_csv:old_way",
            ]
        },
        classifiers=[
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Science/Research",
            "License :: OSI Approved :: Apache Software License",
            "Natural Language :: English",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: 3.11",
            "Topic :: Scientific/Engineering :: Medical Science Apps.",
        ],
        description=(
            "PyTorch-based framework that handles segmentation/regression/classification using various DL architectures for medical imaging."
        ),
        install_requires=requirements,
        license="Apache-2.0",
        long_description=readme,
        long_description_content_type="text/markdown",
        include_package_data=True,
        package_data={"GANDLF": extra_files},
        keywords="semantic, segmentation, regression, classification, data-augmentation, medical-imaging, clinical-workflows, deep-learning, pytorch",
        zip_safe=False,
    )
